/*
 * Copyright (c) 2006 Alvaro Lopes <alvieboy@alvie.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __MODEMDRIVERFAKE_H__
#define __MODEMDRIVERFAKE_H__

#include "modem_driver.h"

struct fake_modem_t
{
    GString *reply;
    GIOFunc reader;
    void *reader_pvt;
    GIOChannel *channel;
    gboolean valid_pin;
    guint32 pin_time;
};

#define REGISTRATION_TIME 40 /* 40 seconds registration */

extern struct modem_driver_t modem_driver_fake;

#endif
