/*
 * Copyright (c) 2006 Alvaro Lopes <alvieboy@alvie.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __CELLMODEM_OPTIONS_H__
#define __CELLMODEM_OPTIONS_H__

#include <glib.h>

typedef struct
{
    gboolean  display_tooltip_info;
    gchar     *modem_device;
    gchar     *modem_driver;
    gchar     *ppp_peer;
/*    gboolean  ask_for_pin;*/
    gint      low_threshold;
    gint      critical_threshold;
    gint      max_quality;
} cellmodem_options_t;

#endif
