/*
 * Copyright (c) 2006 Alvaro Lopes <alvieboy@alvie.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __PREFERENCES_H__
#define __PREFERENCES_H__

#include "cellmodem.h"

typedef struct
{
    GtkWidget *device_entry;
    GtkWidget *driver_entry;
    GtkWidget *low_entry;
    GtkWidget *critical_entry;
    GtkWidget *maxqual_entry;
    GtkWidget *autodetect_dialog;
    GtkWidget *device_combo;
/*    GtkWidget *ask_pin_entry;*/
    cellmodem_t *monitor;
    GList *devices;
} cellmodem_dialog_t;

void
cellmodem_create_options(XfcePanelPlugin *plugin, cellmodem_t *monitor);

#endif
